using System;
using System.Text;
using System.Security.Cryptography;
using System.Configuration;

public class TamperDetector {
    public static string AddTamperDetection(string s) {
        byte[] data = Encoding.UTF8.GetBytes(s);
        byte[] hash = getKeyedHash().ComputeHash(data);
        return Convert.ToBase64String(hash) + '|' + s;

    }

    public static string CheckAndRemoveTamperDetection(string s) {
        int i = s.IndexOf('|');
        if (-1 == i) throw new DataTamperingException("Unexpected format.");
        string prefix = s.Substring(0, i);
        string suffix = s.Substring(i + 1);
        byte[] hash = Convert.FromBase64String(prefix);
        byte[] data = Encoding.UTF8.GetBytes(suffix);
        byte[] computedHash = getKeyedHash().ComputeHash(data);
        if (!isEqual(hash, computedHash))
            throw new DataTamperingException("String has been modified!");
        return suffix;

    }

    public static string GenerateRandomKey() {
        byte[] rnd = new byte[16]; // 128 bits
        new RNGCryptoServiceProvider().GetBytes(rnd);
        return Convert.ToBase64String(rnd);

    }

    static bool isEqual(byte[] a, byte[] b) {
        if (a.Length != b.Length) return false;
        for (int i = 0; i < a.Length; ++i)
            if (a[i] != b[i]) return false;
        return true;
    }

    static HMACSHA1 getKeyedHash() {
        string skey = ConfigurationManager.AppSettings["validationKey"];
        byte[] key = Convert.FromBase64String(skey);
        return new HMACSHA1(key);
    }
}

public class DataTamperingException : Exception {
    public DataTamperingException(string msg) : base(msg) { }
}
